#define _CRT_SECURE_NO_WARNINGS

//#include <unistd.h>
#include <stdio.h>
#include <stdlib.h>
//#include <sys/ioctl.h>
//#include <linux/types.h>
#include <errno.h>
#include <sys/stat.h>
#include <fcntl.h>
//#include <err.h>
#include <string.h>

#include "HIDCommand.h"
#include "INOGENIHID.h"


#define MAXIMUM_FX3_ERROR_CODE (HID_NO_RESPONSE_OR_INVALID - 1)


unsigned char MessageID(void)
{
	unsigned char static _MessageID = 0x01;
	unsigned char temp = _MessageID;
	
	_MessageID++;
	return temp;
}

enum ErrorCode ParseDeviceAnswer(unsigned char *Primitive, unsigned char *Answer, unsigned char *ReturnAnswerData)
{
	int DataLen = 0;
	int i = 0;
	
	//Check Message ID
	if (Primitive[0] != Answer[0]) 
	{
		//Oops this not an answer from the current request
		return HID_NO_RESPONSE_OR_INVALID;
	}
	
	//Check the command it must be the same
	if (Answer[1] != Primitive[1])
	{
		//Oops this not an answer from the current request
		return HID_NO_RESPONSE_OR_INVALID;
	}
	
	if (Answer[2] < 1)
	{
		return HID_NO_RESPONSE_OR_INVALID;
	}
	
	
	//Error code expected
	if (Answer[3] == HID_ACK)
	{
		//Success
	}
	else if (Answer[3] > MAXIMUM_FX3_ERROR_CODE)
	{
		//Invalid
		return HID_NO_RESPONSE_OR_INVALID;
	}
	else
	{
		// FX3 return an error
		return (enum ErrorCode)Answer[3];
	}
	
	DataLen  = Answer[2] - 1; // ok remove the acknoledge byte

	//Ok we have an answer
	if (DataLen)
	{
		// We have an answer to return
		for(i = 0; i < DataLen; i++)
		{
			ReturnAnswerData[i] = Answer[i+4];
		}
	}
	
	return (enum ErrorCode)Answer[3];
}

void BuildGetFX3Register(unsigned char Register, unsigned char *Primitive) 
{ 
	Primitive[0] = MessageID(); // Message ID; manage by the object
	Primitive[1] = HID_DEBUG_RD; // command
	Primitive[2] = 1; // Always length of 1
	Primitive[3] = Register; //Register
}

void BuildSetFX3Register(unsigned char Register, unsigned char Value, unsigned char *Primitive) 
{ 
	Primitive[0] = MessageID(); // Message ID; manage by the object
	Primitive[1] = HID_DEBUG_WR; // command
	Primitive[2] = 2; // Always length of 1
	Primitive[3] = Register; //Register
	Primitive[4] = Value; //Register value
}

void BuildI2Cread(unsigned char I2CAddress, unsigned char Register, unsigned char Len, unsigned char *Primitive)
{
	Primitive[0] = MessageID(); // Message ID; manage by the object
	Primitive[1] = HID_I2C_RD_CMD; // command
	Primitive[2] = 3; // Always length of 1
	Primitive[3] = I2CAddress; //Register
	Primitive[4] = Register; //Register value
	Primitive[5] = Len; //Register value
}


void BuildReadSPIEEPROMFX3(unsigned long Address, unsigned char Len, unsigned char *Primitive)
{
	Primitive[0] = MessageID(); // Message ID; manage by the object
	Primitive[1] = HID_SPI_RD_CMD; // command
	Primitive[2] = 5; // Always length of 5
	Primitive[3] = 0; //SPI_DEV is for future used left at 0
	Primitive[4] = ((Address & 0xFF0000) >> 16); //Address [16:23]
	Primitive[5] = ((Address & 0xFF00) >> 8); //Address [8:15]
	Primitive[6] = ((Address & 0xFF)); //Address [0:7]
	Primitive[7] = Len; //Register value
}

void BuildWriteSPIEEPROMFX3(unsigned long Address, unsigned char Len, unsigned char *Data, unsigned char *Primitive)
{
	int i = 0;
	Primitive[0] = MessageID(); // Message ID; manage by the object
	Primitive[1] = HID_SPI_WR_CMD; // command
	Primitive[2] = Len + 4; // data len + SPI_DEV + 3 bytes of 
	Primitive[3] = 0; //SPI_DEV is for future used left at 0
	Primitive[4] = ((Address & 0xFF0000) >> 16); //Address [16:23]
	Primitive[5] = ((Address & 0xFF00) >> 8); //Address [8:15]
	Primitive[6] = ((Address & 0xFF)); //Address [0:7]
	for (i = 0; i < Len; i++)
	{
		Primitive[7 + i] = Data[i];
	}
}

void BuildEraseEEPROMSectorSPIFX3(unsigned char Sector, unsigned char *Primitive)
{
	int i = 0;
	Primitive[0] = MessageID(); // Message ID; manage by the object
	Primitive[1] = HID_SPI_ERASE_CMD; // command
	Primitive[2] = 2; // 2
	Primitive[3] = 0; //SPI_DEV is for future used left at 0
	Primitive[4] = Sector; 
}

void BuildReadDescriptor(unsigned long Address, unsigned char Len, unsigned char *Primitive)
{
	Primitive[0] = MessageID(); // Message ID; manage by the object
	Primitive[1] = HID_DESCR_RD_CMD; // command
	Primitive[2] = 5; // Always length of 5
	Primitive[3] = 0; //SPI_DEV is for future used left at 0
	Primitive[4] = ((Address & 0xFF0000) >> 16); //Address [16:23]
	Primitive[5] = ((Address & 0xFF00) >> 8); //Address [8:15]
	Primitive[6] = ((Address & 0xFF)); //Address [0:7]
	Primitive[7] = Len; //Register value
}

void BuildWriteDescriptor(unsigned long Address, unsigned char Len, unsigned char *Data, unsigned char *Primitive)
{
	int i = 0;
	Primitive[0] = MessageID(); // Message ID; manage by the object
	Primitive[1] = HID_DESCR_WR_CMD; // command
	Primitive[2] = Len + 4; // data len + SPI_DEV + 3 bytes of 
	Primitive[3] = 0; //SPI_DEV is for future used left at 0
	Primitive[4] = ((Address & 0xFF0000) >> 16); //Address [16:23]
	Primitive[5] = ((Address & 0xFF00) >> 8); //Address [8:15]
	Primitive[6] = ((Address & 0xFF)); //Address [0:7]
	for (i = 0; i < Len; i++)
	{
		Primitive[7 + i] = Data[i];
	}
}


enum ErrorCode GetFX3Register(unsigned char Register, unsigned char *RegisterValue)
{
	unsigned char data_out[INOGENI_REPORT_LEN];
	unsigned char data_in[INOGENI_REPORT_LEN];
	unsigned char data_in_filter[INOGENI_REPORT_LEN];
	int result = 0;
	enum ErrorCode Response;
	
	// Build the primitive
	BuildGetFX3Register(Register, data_out);
	
	result = exchangeReport(data_out, data_in);
	if (result != 0)
	{
		//printf("ERROR! error in report exchange\n");
		return HID_NO_RESPONSE_OR_INVALID;
	}
	else
	{
		// Succces now parse the answer from the device
		Response = ParseDeviceAnswer(data_out, data_in, data_in_filter);
		if(Response == HID_ACK)
		{
			*RegisterValue = data_in_filter[0];
		}
	}
	return Response;
}

enum ErrorCode SetFX3Register(unsigned char Register, unsigned char RegisterValue)
{
	unsigned char data_out[INOGENI_REPORT_LEN];
	unsigned char data_in[INOGENI_REPORT_LEN];
	unsigned char data_in_filter[INOGENI_REPORT_LEN];
	int result = 0;
	enum ErrorCode Response;
	
	// Build the primitive
	BuildSetFX3Register(Register, RegisterValue, data_out);
	
	result = exchangeReport(data_out, data_in);
	if (result != 0)
	{
		//printf("ERROR! error in report exchange\n");
		return HID_NO_RESPONSE_OR_INVALID;
	}
	else
	{
		// Succces now parse the answer from the device
		Response = ParseDeviceAnswer(data_out, data_in, data_in_filter);
	}
	return Response;
}


enum ErrorCode ReadSPIEEPROM(unsigned long Address, unsigned char Len, unsigned char *Data)
{
	unsigned char data_out[INOGENI_REPORT_LEN];
	unsigned char data_in[INOGENI_REPORT_LEN];
	unsigned char data_in_filter[INOGENI_REPORT_LEN];
	int result = 0;
	enum ErrorCode Response;
	int i = 0;
	
	// Build the primitive
	BuildReadSPIEEPROMFX3(Address, Len, data_out);
	
	result = exchangeReport(data_out, data_in);
	if (result != 0)
	{
		//printf("ERROR! error in report exchange\n");
		return HID_NO_RESPONSE_OR_INVALID;
	}
	else
	{
		// Succces now parse the answer from the device
		Response = ParseDeviceAnswer(data_out, data_in, data_in_filter);
		if(Response == HID_ACK)
		{
			for (i = 0; i < Len; i++)
			{
				Data[i] = data_in_filter[i];
			}
		}
	}
	return Response;
}


enum ErrorCode WriteSPIEEPROM(unsigned long Address, unsigned char Len, unsigned char *Data)
{
	unsigned char data_out[INOGENI_REPORT_LEN];
	unsigned char data_in[INOGENI_REPORT_LEN];
	unsigned char data_in_filter[INOGENI_REPORT_LEN];
	int result = 0;
	enum ErrorCode Response;
	int i = 0;
	
	// Build the primitive
	BuildWriteSPIEEPROMFX3(Address, Len, Data, data_out);
	
	result = exchangeReport(data_out, data_in);
	if (result != 0)
	{
		//printf("ERROR! error in report exchange\n");
		return HID_NO_RESPONSE_OR_INVALID;
	}
	else
	{
		// Succces now parse the answer from the device
		Response = ParseDeviceAnswer(data_out, data_in, data_in_filter);
	}
	return Response;
}

enum ErrorCode EraseSPISector(unsigned char Sector)
{
	unsigned char data_out[INOGENI_REPORT_LEN];
	unsigned char data_in[INOGENI_REPORT_LEN];
	unsigned char data_in_filter[INOGENI_REPORT_LEN];
	int result = 0;
	enum ErrorCode Response;
	int i = 0;
	
	
	// Build the primitive
	BuildEraseEEPROMSectorSPIFX3(Sector, data_out);
	
	result = exchangeReport(data_out, data_in);
	if (result != 0)
	{
		//printf("ERROR! error in report exchange\n");
		return HID_NO_RESPONSE_OR_INVALID;
	}
	else
	{
		//printf("Erase EEPROM Sector %d success\n", Sector);
		// Succces now parse the answer from the device
		Response = ParseDeviceAnswer(data_out, data_in, data_in_filter);
	}
	return Response;
} 

/*******************************************************************/
/************************High Level FCT******************************/
/*******************************************************************/

void DisplayHIDErrorCodeMeaning(enum ErrorCode code, enum RequestType AccessType)
{
	switch (code)
	{
	case HID_ACK:
		printf("USB3 device return: Acknoledge.\n");
		break;
	case HID_BAD_ADDR:
		printf("USB3 device return: Bad Address.\n");
		break;
	case HID_BAD_CMD_NUM:
		printf("USB3 device return: Invalid command number.\n");
		break;
	case HID_BAD_LENGTH:
		printf("USB3 device return: Invalid Primitive Length.\n");
		break;
	case HID_BUSY:
		printf("USB3 device return: Busy.\n");
		break;
	case HID_NO_RESPONSE_OR_INVALID:
		printf("USB3 device return: No response receive from HID device or answer is invalid!\n");
		break;
	case HID_PC_EXCEPTION:
		printf("USB3 device return: Application has encounter an unknown error\n");
		break;
	case HID_TOO_BAD:
		printf("USB3 device return: Too Bad\n");
		break;
	case HID_BAD_DATA_LENGTH:
		printf("USB3 device return: Bad Data request Length!\n");
		break;
	case HID_SPI_FAIL:
		printf("USB3 device return: SPI communication has failed!\n");
		break;
	case HID_I2C_NO_ANSWER:
		printf("USB3 device return: No answer received from target I2C device!\n");
		break;
	case HID_I2C_COMM_FAILED:
		printf("USB3 device return: I2C communication has failed!\n");
		break;
	case HID_NOT_SUPPORTED:
		printf("USB3 device return: This feature is currently not supported!\n");
		break;
	case HID_REG_ACCES_NOT_SUPPORTED:
		if (AccessType == Read)
		{
			printf("USB3 device return: This register cannot be read!\n");
		}
		else if(AccessType == Write)
		{
			printf("USB3 device return: This register cannot be written.\n");
		}
		else
		{
			printf("This request access type is not supported.\n");
		}
		break;
	case HID_FPGA_NOT_CONFIGURED:
		printf("USB3 device return: The FPGA does not appear to be configured!\n");
		break;
	default:
		printf("USB3 device return: the application has encounter an unknown error\n");
		break;
	}
}

enum ErrorCode GetFx3Version(unsigned char *Major, unsigned char *Minor)
{
	unsigned char Value = 0;
	enum ErrorCode resultcode;
	
	*Major = 0;
	*Minor = 0;
	
	resultcode = GetFX3Register(0x00, &Value);
	if (resultcode == HID_ACK)
	{
		*Major = Value;
		//printf("Register 0x%02X = 0x%02X\n", 0x00, Value);
	}
	else
	{
		//printf("Return code = 0x%02X\n", resultcode);
		return resultcode;
	}
	resultcode = GetFX3Register(0x01, &Value);
	if (resultcode == HID_ACK)
	{
		*Minor = Value;
		//printf("Register 0x%02X = 0x%02X\n", 0x01, Value);
	}
	else
	{
		//printf("Return code = 0x%02X\n", resultcode);
		return resultcode;
	}
	return HID_ACK;
}

enum ErrorCode isFx3Initialize(unsigned char *isInit)
{
	unsigned char Value = 0;
	enum ErrorCode resultcode;
	
	*isInit = 0;
	
	resultcode = GetFX3Register(0x04, &Value);
	if (resultcode == HID_ACK)
	{
		if(Value & 0x02)
		{
			*isInit = 0x01;
		}
		//printf("Register 0x%02X = 0x%02X\n", 0x00, Value);
	}
	else
	{
		//printf("Return code = 0x%02X\n", resultcode);
		return resultcode;
	}
	
	return HID_ACK;
}

enum ErrorCode GetFPGAVersion(unsigned char *Major, unsigned char *Minor)
{
	unsigned char Value = 0;
	enum ErrorCode resultcode;
	
	*Major = 0;
	*Minor = 0;
	
	resultcode = GetFX3Register(0x03, &Value);
	if (resultcode == HID_ACK)
	{
		*Major = ((Value & 0xF0) >> 4);
		*Minor = (Value & 0x0F);
		//printf("Register 0x%02X = 0x%02X\n", 0x00, Value);
	}
	else
	{
		//printf("Return code = 0x%02X\n", resultcode);
		return resultcode;
	}

	return HID_ACK;
}


enum ErrorCode GetEDIDVersion(unsigned char *Version)
{
	unsigned char Value = 0;
	enum ErrorCode resultcode;
	
	*Version = 0;
	
	resultcode = GetFX3Register(12, &Value);
	if (resultcode == HID_ACK)
	{
		*Version = Value & 0xFF;
		
		//printf("Register 0x%02X = 0x%02X\n", 0x00, Value);
	}
	else
	{
		//printf("Return code = 0x%02X\n", resultcode);
		return resultcode;
	}

	return HID_ACK;
}

// Process has done i out of n rounds,
// and we want a bar of width w and resolution r.
static inline void loadBar(int x, int n, int r, int w)
{
	int i,j;
	// Only update r times.
	if ( x % (n/r) != 0 ) return;

	// Calculuate the ratio of complete-to-incomplete.
	float ratio = x/(float)n;
	int   c     = ratio * w;

	// Show the percentage complete.
	printf("%3d%% [", (int)(ratio*100) );

	// Show the load bar.
	for (i=0; i<c; i++)
	printf("=");

	for (j=c; j<w; j++)
	printf(" ");

	// ANSI Control codes to go back to the
	// previous line and clear it.
	//printf("]\n\033[F\033[J");
	printf("]\r");
	fflush(stdout);
}



enum ErrorCode SPILocationErase(struct SPIEEPROMLocation *SPILocation)
{
	int i = 0;
	enum ErrorCode res = HID_NO_RESPONSE_OR_INVALID;	
	int TotalSector = 0;
	int SectorErased = 0;
	
	//Erase
	TotalSector = (SPILocation->LastSector - SPILocation->FirstSector + 1);
	
	printf("Erasing...\n"); // init percentage display
	loadBar(0, TotalSector, TotalSector, 10);
	for (i = SPILocation->FirstSector; i <= SPILocation->LastSector; i++)
	{
		
		res = HID_NO_RESPONSE_OR_INVALID;
		while (res != HID_ACK)
		{
			res = EraseSPISector(i);
			if (res != HID_ACK && res != HID_BUSY)
			{
				printf("Erase Sector %d failed\n", i); // init percentage display
				DisplayHIDErrorCodeMeaning(res, Write);
				return res;
			} 
		}
		
		SectorErased++;		 	  
		
		loadBar(SectorErased, TotalSector, TotalSector, 10);

		sleep(3000);
	}
	loadBar(TotalSector, TotalSector, TotalSector, 10);


	return HID_ACK;
}


enum ErrorCode SPILocationProgramFromFile(struct SPIEEPROMLocation *SPILocation,  char *filename)
{
	int fd = -1;
	struct stat st;
	long totalFilesize = -1;
	unsigned char Data[MAXIMUM_SPI_WRITE_SIZE];
	enum ErrorCode Result;
	unsigned long Address = SPILocation->FirstAddressActive;
	unsigned long checksum = 0;
	int i = 0;
	unsigned long TotalSize = 0;
	int RequestLen = 0;
	
	/* Open the file */
	fd = open(filename, O_RDONLY);
	if (fd == -1)
	{
		//close(fd);
		printf("Error: Unable to open the selected file!\n");
		return HID_PC_EXCEPTION;
	}

	// Set the errno to default value
	errno = 0;

	/* We need the file size to verfiy the file is not too large */
	if (fstat(fd, &st))
	{
		//printf("Error: fstat error: [%s]\n", strerror(errno));
		close(fd);
		return HID_PC_EXCEPTION;
	}
	totalFilesize = st.st_size;


	//printf("File size = %ld bytes\n", totalFilesize);


	/* Check that the file size is within the possible range */
	// TODO check for a minimum?
	if(totalFilesize > SPILocation->Size)
	{
		// We have a problem the file is larger than the available space it won't work don't lose time comparing the fitting part
		printf("Error: The file is larger than the available space in EEPROM!!!\n");
		close(fd);
		return HID_PC_EXCEPTION;
	}

	printf("\nWriting...\n"); // init percentage display
	loadBar(0, totalFilesize, totalFilesize, 10);
	if (SPILocation->Destination == FPGA)
	{
		// Extra step for the FPGA we need to write the size in the first page
		Data[0] = (totalFilesize & 0xFF);
		Data[1] = ((totalFilesize & 0xFF00) >> 8);
		Data[2] = ((totalFilesize & 0xFF0000) >> 16);
		Data[3] = ((totalFilesize & 0xFF000000) >> 24);
		for (i = 0; i < 4; i++)
		{
			checksum += Data[i];
		}
		checksum = checksum % 256;
		checksum = (256 - checksum) % 256;
		Data[4] = checksum;
		
		//Write the len in the first page
		Result = HID_NO_RESPONSE_OR_INVALID;
		while (Result !=  HID_ACK)
		{
			Result = WriteSPIEEPROM(Address, 5, Data);
			if ((Result != HID_ACK) && (Result != HID_BUSY))
			{
				close(fd);
				DisplayHIDErrorCodeMeaning(Result, Write);
				return Result;
			}
		}
		Address += EEPROM_FX3_BYTE_PER_PAGE;                   
	}

	do
	{
		RequestLen = read(fd, Data, MAXIMUM_SPI_WRITE_SIZE);
		if(RequestLen == -1)
		{
			if (errno == EINTR)
			{
				continue;
			}
			else
			{
				close(fd);
				printf("Error reading the file!\n");
				return HID_PC_EXCEPTION;
			}
		}
		
		if (RequestLen > 0)
		{
			Result = HID_NO_RESPONSE_OR_INVALID;
			while (Result != HID_ACK)
			{
				Result = WriteSPIEEPROM(Address, RequestLen, Data);
				if (Result != HID_ACK && Result != HID_BUSY)
				{	
					DisplayHIDErrorCodeMeaning(Result, Write);
					close(fd);
					return Result;
				}
			}
			Address +=  (unsigned long)RequestLen;
			TotalSize +=  (unsigned long)RequestLen;
		}
		loadBar(TotalSize, totalFilesize, totalFilesize, 10);
	} while(RequestLen > 0 && TotalSize < totalFilesize);

	printf("\n");
	close(fd);
	return HID_ACK;
}


enum ErrorCode SPILocationVerifyFromFile(struct SPIEEPROMLocation *SPILocation,  char *filename)
{
	int fd = -1;fd = open(filename, O_RDONLY);
	struct stat st;
	long totalFilesize = -1;
	unsigned char DataFile[MAXIMUM_SPI_READ_SIZE];
	unsigned char DataEEPROM[MAXIMUM_SPI_READ_SIZE];
	enum ErrorCode Result;
	unsigned long Address = SPILocation->FirstAddressActive;
	unsigned long checksum = 0;
	int i = 0;
	unsigned long TotalSize = 0;
	int RequestLen = 0;
	
	unsigned long EEPROMSize = 0;
	
	/* Open the file */
	fd = open(filename, O_RDONLY);
	if (fd == -1)
	{
		//close(fd);
		printf("Error: Unable to open the selected file!\n");
		return HID_PC_EXCEPTION;
	}

	// Set the errno to default value
	errno = 0;

	/* We need the file size to verfiy the file is not too large */
	if (fstat(fd, &st))
	{
		//printf("Error: fstat error: [%s]\n", strerror(errno));
		close(fd);
		return HID_PC_EXCEPTION;
	}
	totalFilesize = st.st_size;


	//printf("File size = %ld bytes\n", totalFilesize);


	/* Check that the file size is within the possible range */
	// TODO check for a minimum?
	if(totalFilesize > SPILocation->Size)
	{
		// We have a problem the file is larger than the available space it won't work don't lose time comparing the fitting part
		printf("Error: The file is larger than the available space in EEPROM!!!\n");
		close(fd);
		return HID_PC_EXCEPTION;
	}


	printf("Verifying...\n"); // init percentage display
	loadBar(0, totalFilesize, totalFilesize, 10);

	if (SPILocation->Destination == FPGA)
	{
		// We need to check the bit stream size at the beginning of FPGA section
		Result = HID_NO_RESPONSE_OR_INVALID;
		while (Result != HID_ACK)
		{
			Result = ReadSPIEEPROM(Address, 5, DataEEPROM);
			if (Result != HID_ACK && Result != HID_BUSY)
			{	
				DisplayHIDErrorCodeMeaning(Result, Write);
				close(fd);
				return Result;
			}
		}
		
		
		//Verify the size checksum
		for(i = 0; i < 4; i++)
		{
			checksum += DataEEPROM[i];
		}
		checksum = ((256 - checksum) % 256);

		if(checksum != DataEEPROM[4])
		{
			printf("The checksum is incorrect!");
			close(fd);
			return HID_PC_EXCEPTION;
		}

		EEPROMSize = ((unsigned long)DataEEPROM[0]) | ((unsigned long)DataEEPROM[1] << 8) | ((unsigned long)DataEEPROM[2] << 16) | ((unsigned long)DataEEPROM[3] << 24);

		if(EEPROMSize != totalFilesize)
		{
			printf("The bit stream size is incorrect!");
			close(fd);
			return HID_PC_EXCEPTION;
		}
		
		// we skip the first page for the FPGA as it contain the size
		Address += EEPROM_FX3_BYTE_PER_PAGE;                   
	}


	do
	{
		//Read form the file the size of one eeprom read command
		if(( RequestLen = read(fd, DataFile, MAXIMUM_SPI_READ_SIZE)) == -1)
		{
			if (errno == EINTR)
			{
				continue;
			}
			else
			{
				close(fd);
				printf("Error reading the file!\n");
				return HID_PC_EXCEPTION;
			}

		}
		
		if (RequestLen > 0)
		{
			Result = HID_NO_RESPONSE_OR_INVALID;
			while (Result != HID_ACK)
			{
				Result = ReadSPIEEPROM(Address, (unsigned char)RequestLen, DataEEPROM);
				if (Result != HID_ACK && Result != HID_BUSY)
				{	
					DisplayHIDErrorCodeMeaning(Result, Write);
					close(fd);
					return Result;
				}
			}
			
			// Ok compare the data from the EEPROM and from the FPGA
			for(i = 0; i < RequestLen; i++)
			{
				if(DataEEPROM[i] != DataFile[i])
				{
					printf("\nError at Address 0x%X, the data read from the EEPROM 0x%02X is different than the data from the file 0x%02X\n",Address+i, DataEEPROM[i], DataFile[i]);
					close(fd);
					return HID_PC_EXCEPTION;
				}
			}
			//Data is Ok continue
			
			
			Address +=  (unsigned long)RequestLen;
			TotalSize +=  (unsigned long)RequestLen;
		}
		loadBar(TotalSize, totalFilesize, totalFilesize, 10);
	}while(RequestLen > 0 && TotalSize < totalFilesize);

	loadBar(totalFilesize, totalFilesize, totalFilesize, 10);
	printf("\n");
	close(fd);
	return HID_ACK;
}

enum ErrorCode StopFPGADownload()
{
	unsigned char Value = 0;
	enum ErrorCode Response = GetFX3Register(GeneralRegister, &Value);
	
	if (Response == HID_ACK)
	{
		Value |= REG_GENERAL_STOP_FPGA_DOWNLOAD;
	}
	else
	return Response;
	
	Response = SetFX3Register(GeneralRegister, Value);
	return Response;
}

enum ErrorCode SPIFPGAProgram(char *filename)
{
	enum ErrorCode Result = HID_PC_EXCEPTION;
	
	Result = StopFPGADownload();
	
	if(Result == HID_ACK)
	{
		//Good
	}
	else
	{
		DisplayHIDErrorCodeMeaning(Result, Unspecified);
		return Result;
	}
	
	printf("Updating The FPGA, this may take a few minutes...\n");
	printf("Please wait...\n");

	Result = SPILocationErase(&FPGABitStreamInM25P80);
	if(Result != HID_ACK)
	{
		return Result;
	}
	
	Result = SPILocationProgramFromFile(&FPGABitStreamInM25P80, filename);
	if (Result != HID_ACK)
	{
		return Result;
	}
	return Result;
}

enum ErrorCode SPIFPGAVerify(char *filename)
{
	enum ErrorCode Result = HID_PC_EXCEPTION;
	
	Result = StopFPGADownload();
	
	if(Result == HID_ACK)
	{
		//Good
	}
	else
	{
		DisplayHIDErrorCodeMeaning(Result, Unspecified);
		return Result;
	}
	
	printf("Verifying The FPGA, this may take a few minutes...\n");
	printf("Please wait...\n");
	Result = SPILocationVerifyFromFile(&FPGABitStreamInM25P80,  filename);
	
	return Result;
}

enum ErrorCode SPIFX3Program(char *filename)
{
	enum ErrorCode Result = HID_PC_EXCEPTION;
	
	printf("Updating The FX3, this may take a few minutes...\n");
	printf("Please wait...\n");
	Result = SPILocationErase(&FX3FirmwareInM25P80);
	if(Result != HID_ACK)
	{
		return Result;
	}
	
	Result = SPILocationProgramFromFile(&FX3FirmwareInM25P80, filename);
	if(Result != HID_ACK)
	{
		return Result;
	}
	return Result;
}

enum ErrorCode SPIFX3Verify(char *filename)
{
	enum ErrorCode Result = HID_PC_EXCEPTION;
	
	printf("Verifying The FX3, this may take a few minutes...\n");
	printf("Please wait...\n");
	
	Result = SPILocationVerifyFromFile(&FX3FirmwareInM25P80, filename);
	if(Result != HID_ACK)
	{
		return Result;
	}
	return Result;
}


enum ErrorCode SPIEDIDProgram(char *filename)
{
	enum ErrorCode Result = HID_PC_EXCEPTION;
	
	
	printf("Updating The EDID, this may take several seconds...\n");
	printf("Please wait...\n");
	Result = SPILocationErase(&EDIDInM25P80);
	if(Result != HID_ACK)
	{
		return Result;
	}

	Result = SPILocationProgramFromFile(&EDIDInM25P80, filename);
	if(Result != HID_ACK)
	{
		return Result;
	}

	return Result;
}

enum ErrorCode SPIEDIDVerify(char *filename)
{
	enum ErrorCode Result = HID_PC_EXCEPTION;
	
	
	printf("Verifying The EDID, this may take several seconds...\n");
	printf("Please wait...\n");
	
	Result = SPILocationVerifyFromFile(&EDIDInM25P80, filename);
	if(Result != HID_ACK)
	{
		return Result;
	}

	return Result;
}

void FX3Reset(void)
{
	unsigned char data_out[INOGENI_REPORT_LEN];
	int result = 0;
	enum ErrorCode Response;
	
	// Build the primitive
	BuildSetFX3Register(0xFF, 0x01, data_out);
	SendReport(data_out);
	return;
}

enum ErrorCode GetInputVideoFormat(struct VideoFormat *format)
{
	unsigned char Value = 0;
	enum ErrorCode Response;
	
	format->ActiveLinesF0 = 0;
	format->ActivesPixel = 0;
	format->FrameRate = 0;
	format->Locked = 0;
	
	
	// Get video lock
	Response = GetFX3Register(REG_VIDEO_GENERAL, &Value);
	
	if (Response == HID_ACK)
	{
		// bit 0 is for lock
		if(Value & 0x01)
		{
			format->Locked = 1;
		}
	}
	else
	return Response;
	
	
	
	// Get video width MSB
	Response = GetFX3Register(REG_WIDTH_MSB, &Value);
	
	if (Response == HID_ACK)
	{
		format->ActivesPixel = ((unsigned short)Value << 8);
	}
	else
	return Response;
	
	// Get video width LSB
	Response = GetFX3Register(REG_WIDTH_LSB, &Value);
	
	if (Response == HID_ACK)
	{
		format->ActivesPixel |= ((unsigned short)Value);
	}
	else
	return Response;
	
	// Get video height MSB
	Response = GetFX3Register(REG_HEIGHT_MSB, &Value);
	
	if (Response == HID_ACK)
	{
		format->ActiveLinesF0 = ((unsigned short)Value << 8);
	}
	else
	return Response;
	
	
	REG_FRAME_RATE_MSB,
	REG_FRAME_RATE_LSB,
	
	// Get video height LSB
	Response = GetFX3Register(REG_HEIGHT_LSB, &Value);
	
	if (Response == HID_ACK)
	{
		format->ActiveLinesF0 |= ((unsigned short)Value );
	}
	else
	return Response;
	
	// Get video frame rate msb *100
	Response = GetFX3Register(REG_FRAME_RATE_MSB, &Value);
	
	if (Response == HID_ACK)
	{
		format->FrameRate = ((unsigned short)Value << 8);
	}
	else
	return Response;

	// Get video frame rate lsb *100
	Response = GetFX3Register(REG_FRAME_RATE_LSB, &Value);
	
	if (Response == HID_ACK)
	{
		format->FrameRate |= (unsigned short)Value;
	}
	else
	return Response;
	
	return Response;
}

enum ErrorCode ReadDescriptor(unsigned char *Data)
{
	unsigned char data_out[INOGENI_REPORT_LEN];
	unsigned char data_in[INOGENI_REPORT_LEN];
	unsigned char data_in_filter[INOGENI_REPORT_LEN];
	int result = 0;
	enum ErrorCode Response;
	int i = 0;

	// Build the primitive
	BuildReadDescriptor(0, INOGENI_REPORT_LEN, data_out);

	result = exchangeReport(data_out, data_in);
	if (result != 0)
	{
		//printf("ERROR! error in report exchange\n");
		return HID_NO_RESPONSE_OR_INVALID;
	}
	else
	{
		// Succces now parse the answer from the device
		Response = ParseDeviceAnswer(data_out, data_in, data_in_filter);
		if (1)
		{
			for (i = 0; i < INOGENI_REPORT_LEN; i++)
			{
				Data[i] = data_in_filter[i];
			}
		}
	}
	return Response;
}

enum ErrorCode WriteDescriptor(unsigned char *Data)
{
	unsigned char data_out[INOGENI_REPORT_LEN];
	unsigned char data_in[INOGENI_REPORT_LEN];
	unsigned char data_in_filter[INOGENI_REPORT_LEN];
	int result = 0;
	enum ErrorCode Response;
	int i = 0;

	// Build the primitive
	BuildWriteDescriptor(0, INOGENI_REPORT_LEN, Data, data_out);

	result = exchangeReport(data_out, data_in);
	if (result != 0)
	{
		//printf("ERROR! error in report exchange\n");
		return HID_NO_RESPONSE_OR_INVALID;
	}
	else
	{
		// Succces now parse the answer from the device
		Response = ParseDeviceAnswer(data_out, data_in, data_in_filter);
	}
	return Response;
}



