/* 
* File:   HIDCommand.h
* Author: Jerome
*
* Created on November 22, 2013, 3:38 PM
*/

#ifndef HIDCOMMAND_H
#define	HIDCOMMAND_H

#ifdef	__cplusplus
extern "C" {
#endif

#define MAXIMUM_FX3_REGISTER_INDEX						3
#define MAXIMUM_SPI_READ_SIZE							59
#define MAXIMUM_SPI_WRITE_SIZE							56
#define MAXIMUM_FPGA_READ_SIZE							58
#define MAXIMUM_FPGA_WRITE_SIZE							59
	//#define MAXIMUM_FX3_ERROR_CODE  = ErrorCode.HID_NO_RESPONSE_OR_INVALID - 1
#define SIZE_SPI_EEPROM									0x80000

#define EEPROM_FX3_BYTE_PER_PAGE						256


	//OK SPI EEPROM Config

	// SPI EEPROM M25P40HIDCommand.c:470:18: note: expected ‘struct SPIEEPROMLocation *’ but argument is of type ‘struct SPIEEPROMLocation (*)[1]’
#define M25P40_FX3_FIRST_PAGE							0
#define M25P40_FX3_LAST_PAGE							767
#define M25P40_FX3_FIRST_SECTOR							0
#define M25P40_FX3_LAST_SECTOR							2
#define M25P40_FX3_SIZE									196608 //0x30000 or 192KByte or 1572864 bits
#define M25P40_FX3_START_ADDRESS						0x0
	// FPGA
#define M25P40_FPGA_FIRST_PAGE							768
#define M25P40_FPGA_LAST_PAGE							1792
#define M25P40_FPGA_SIZE								262144		//0x40000 or 256KByte or 2097152 bits
#define M25P40_FPGA_FIRST_SECTOR						3
#define M25P40_FPGA_LAST_SECTOR							6
#define M25P40_FPGA_START_ADDRESS						0x30000

	// EDID
#define M25P40_EDID_FIRST_PAGE							1793
#define M25P40_EDID_LAST_PAGE							2047
#define M25P40_EDID_SIZE								256 //0x100  or 2048 bits
#define M25P40_EDID_SECTOR_SIZE							65536 //0x10000  or 524288 bits
#define M25P40_EDID_FIRST_SECTOR						7
#define M25P40_EDID_LAST_SECTOR							7
#define M25P40_EDID_START_ADDRESS						0x7FF00 //The EDID is actually the last 256 bytes of the EEPROM

	// SPI EEPROM M25P80
#define M25P80_FX3_FIRST_PAGE							0
#define M25P80_FX3_LAST_PAGE							767
#define M25P80_FX3_FIRST_SECTOR							0
#define M25P80_FX3_LAST_SECTOR							2
#define M25P80_FX3_SIZE									196608 //0x30000 or 192KByte or 1572864 bits
#define M25P80_FX3_START_ADDRESS						0x0
	// FPGA
#define M25P80_FPGA_FIRST_PAGE							768
#define M25P80_FPGA_LAST_PAGE							3584
#define M25P80_FPGA_SIZE								786432 //0xB0000 or 704KByte or 5767168 bits
#define M25P80_FPGA_FIRST_SECTOR						3
#define M25P80_FPGA_LAST_SECTOR							14
#define M25P80_FPGA_START_ADDRESS						0x30000
	// EDID
#define M25P80_EDID_FIRST_PAGE							3585
#define M25P80_EDID_LAST_PAGE							4095
#define M25P80_EDID_SIZE								256 //0x100  or 2048 bits
#define M25P80_EDID_SECTOR_SIZE							65536 //0x10000  or 524288 bits
#define M25P80_EDID_FIRST_SECTOR						15
#define M25P80_EDID_LAST_SECTOR							15
#define M25P80_EDID_START_ADDRESS						0xFEF00 //The EDID is actually the last 256 bytes of the EEPROM

	// HDMI Rx
#define AVD7612_I2C_ADDRESS_IO							0x98
#define AVD7612_I2C_ADDRESS_CEC							0x80
#define AVD7612_I2C_ADDRESS_INFO						0x7C
#define AVD7612_I2C_ADDRESS_DPLL						0x4C
#define AVD7612_I2C_ADDRESS_KSV							0x64
#define AVD7612_I2C_ADDRESS_EDID						0x6C
#define AVD7612_I2C_ADDRESS_HDMI						0x68
#define AVD7612_I2C_ADDRESS_CP							0x44

	//General Register
#define REG_GENERAL_STOP_FPGA_DOWNLOAD					0x1

	//Line rate
	/*1920*1080*/
#define LINE_RATE_1080p60								67500
#define LINE_RATE_1080p59								67430
#define LINE_RATE_1080p50								56250
#define LINE_RATE_1080p30								33750
#define LINE_RATE_1080p29								33720
#define LINE_RATE_1080p25								28130
#define LINE_RATE_1080p24								27000
#define LINE_RATE_1080p23								26980

	/*1080i*/
#define LINE_RATE_1080i60								33750
#define LINE_RATE_1080i59								33720
#define LINE_RATE_1080i50								28130

	/* 720p */
#define LINE_RATE_720p60								45000
#define LINE_RATE_720p59								44960
#define LINE_RATE_720p50								37500
#define LINE_RATE_720p30								22500
#define LINE_RATE_720p29								22480
#define LINE_RATE_720p25								18750
#define LINE_RATE_720p24								18000
#define LINE_RATE_720p23								17990

enum EEPROM
{
	M25P40 = 0,
	M25P80
};

enum SPILocationDesc
{
	FX3 = 0,
	FPGA,
	EDID
};

enum HDMIRxMap
{
	IO = 0,
	CEC,
	INFO,
	DPLL,
	KSV,
	MAPEDID,
	HDMI,
	CP
};


enum Command
{
	HID_I2C_WR_CMD = 0,
	HID_I2C_RD_CMD,
	HID_SPI_WR_CMD,
	HID_SPI_RD_CMD,
	HID_SPI_ERASE_CMD,
	HID_SET_BOOT_CMD,
	HID_FPGA_WR_CMD,
	HID_FPGA_RD_CMD,
	HID_DEBUG_WR,
	HID_DEBUG_RD,
	HID_UART_WR,
	HID_UART_RD,
	HID_SDI_WR_CMD,
	HID_SDI_RD_CMD,
	HID_DESCR_WR_CMD,
	HID_DESCR_RD_CMD
};


enum ErrorCode
{
	HID_ACK = 0,
	HID_BUSY,
	HID_BAD_CMD_NUM,
	HID_BAD_LENGTH,
	HID_BAD_ADDR,
	HID_TOO_BAD,
	HID_BAD_DATA_LENGTH,
	HID_SPI_FAIL,
	HID_I2C_NO_ANSWER, //the target I2C device did not responde
	HID_I2C_COMM_FAILED, //Teh I2C communication failed the reason are not given
	HID_NOT_SUPPORTED,  // The current command is valid but module is removed from code with define
	HID_REG_ACCES_NOT_SUPPORTED, 
	HID_FPGA_NOT_CONFIGURED,
	//NN FX3 error code must come before this line
	HID_NO_RESPONSE_OR_INVALID, // this a error code for PC only: No response for current primitive
	HID_PC_EXCEPTION //this a error code for PC only: No response for current primitive
};

enum RequestType
{
	Read = 0,
	Write,
	Unspecified
};

enum SPIPortSelect
{
	SPI_EEPROM_PORT = 0,
	SPI_FPGA_PORT
};


enum FX3Register
{
	VersionMajor,
	VersionMinor,
	SPIType,
	FPGAVersion,
	GeneralRegister,
	REG_WIDTH_MSB,
	REG_WIDTH_LSB,
	REG_HEIGHT_MSB,
	REG_HEIGHT_LSB,
	REG_FRAME_RATE_MSB,
	REG_FRAME_RATE_LSB,
	REG_VIDEO_GENERAL, //     ' //bit 0 1 == video locked
	EDID_VERSION,
	REG_STATUS,
	REG_WIDTH_MSB_2,
	REG_WIDTH_LSB_2,
	REG_HEIGHT_MSB_2,
	REG_HEIGHT_LSB_2,
	REG_FRAME_RATE_MSB_2,
	REG_FRAME_RATE_LSB_2,
	REG_VIDEO_GENERAL_2 , //     ' //bit 0 1 == video locked
	REG_DEBUG_REG_7,
	REG_DEBUG_REG_8,
	REG_DEBUG_REG_9,
	REG_DEBUG_REG_10,
	REG_GET_GPIF_STATE,
	REG_GET_GPIF_STATE_RETURN,
	REG_RESET_MEMORY,
	REG_DEBUG_A,
	REG_DEBUG_B,
	REG_FLOW_CTRL,
	REG_LINE_RATE_MSB,
	REG_LINE_RATE,
	REG_LINE_RATE_LSB,

	REG_UVC_FRAMERATE_1,
	REG_UVC_FRAMERATE_2,
	REG_UVC_FRAMERATE_3,
	REG_UVC_FRAMERATE_4,

	REG_UVC_PIX_FORMAT,
	REG_UVC_FRAME,
	REG_NO_INPUT_COLOR,
	REG_AUDIO_SELECTOR,

	REG_USB_PHY_ERR_MSB,
	REG_USB_PHY_ERR_LSB,
	REG_USB_LINK_ERR_MSB,
	REG_USB_LINK_ERR_LSB,
	REG_USB_PHY_ERR_CNT_MSB,
	REG_USB_PHY_ERR_CNT_LSB,
	REG_USB_LINK_ERR_CNT_MSB,
	REG_USB_LINK_ERR_CNT_LSB,
	REG_USB_PHY_TRAIN_MSB,
	REG_USB_PHY_TRAIN_LSB,
	REG_USB_PHY_CRC32_MSB,
	REG_USB_PHY_CRC32_LSB,
	REG_USB_PHY_CRC16_MSB,
	REG_USB_PHY_CRC16_LSB,
	REG_USB_PHY_CRC5_MSB,
	REG_USB_PHY_CRC5_LSB,
	REG_USB_PHY_DISP_MSB,
	REG_USB_PHY_DISP_LSB,
	REG_USB_PHY_UFW_MSB,
	REG_USB_PHY_UFW_LSB,
	REG_USB_PHY_OFW_MSB,
	REG_USB_PHY_OFW_LSB,
	REG_USB_PHY_DECODE_MSB,
	REG_USB_PHY_DECODE_LSB,
	REG_USB_LNK_14_MSB,
	REG_USB_LNK_14_LSB,
	REG_USB_LNK_13_MSB,
	REG_USB_LNK_13_LSB,
	REG_USB_LNK_12_MSB,
	REG_USB_LNK_12_LSB,
	REG_USB_LNK_11_MSB,
	REG_USB_LNK_11_LSB,
	REG_USB_LNK_10_MSB,
	REG_USB_LNK_10_LSB,
	REG_USB_LNK_9_MSB,
	REG_USB_LNK_9_LSB,
	REG_USB_LNK_8_MSB,
	REG_USB_LNK_8_LSB,
	REG_USB_LNK_7_MSB,
	REG_USB_LNK_7_LSB,
	REG_USB_LNK_6_MSB,
	REG_USB_LNK_6_LSB,
	REG_USB_LNK_5_MSB,
	REG_USB_LNK_5_LSB,
	REG_USB_LNK_4_MSB,
	REG_USB_LNK_4_LSB,
	REG_USB_LNK_3_MSB,
	REG_USB_LNK_3_LSB,
	REG_USB_LNK_2_MSB,
	REG_USB_LNK_2_LSB,
	REG_USB_LNK_1_MSB,
	REG_USB_LNK_1_LSB,
	REG_USB_LNK_0_MSB,
	REG_USB_LNK_0_LSB,


	REG_SERIAL_NUMBER_1,
	REG_SERIAL_NUMBER_2,
	REG_SERIAL_NUMBER_3,
	REG_SERIAL_NUMBER_4,

	REG_SELECTED_ANALOG_INPUT,

	REG_DEVICE_ID,

	REG_UART_READ_ID,
	REG_UART_READ_SIZE,
	REG_UART_BYTE_0,
	REG_UART_BYTE_1,
	REG_UART_BYTE_2,
	REG_UART_BYTE_3,
	REG_UART_BYTE_4,
	REG_UART_BYTE_5,
	REG_UART_BYTE_6,
	REG_UART_BYTE_7,
	REG_UART_BYTE_8,
	REG_UART_BYTE_9,
	REG_UART_BYTE_10,
	REG_UART_BYTE_11,
	REG_UART_BYTE_12,
	REG_UART_BYTE_13,
	REG_UART_BYTE_14,
	REG_UART_BYTE_15,
		
	REG_PIP_LAYOUT,
	REG_PIP0_TOP_PIX_LSB,
	REG_PIP0_TOP_PIX_MSB,
	REG_PIP0_TOP_LINE_LSB,
	REG_PIP0_TOP_LINE_MSB,
	REG_PIP0_BOT_PIX_LSB,
	REG_PIP0_BOT_PIX_MSB,
	REG_PIP0_BOT_LINE_LSB,
	REG_PIP0_BOT_LINE_MSB,
	REG_PIP1_TOP_PIX_LSB,
	REG_PIP1_TOP_PIX_MSB,
	REG_PIP1_TOP_LINE_LSB,
	REG_PIP1_TOP_LINE_MSB,
	REG_PIP1_BOT_PIX_LSB,
	REG_PIP1_BOT_PIX_MSB,
	REG_PIP1_BOT_LINE_LSB,
	REG_PIP1_BOT_LINE_MSB,
	REG_KEYPAD_VERSION,
	REG_RTC_SECOND,
	REG_RTC_MINUTE,
	REG_RTC_HOUR,
	REG_RTC_DAY,
	REG_RTC_MONTH,
	REG_RTC_YEAR,
	REG_IP_ADDR_0,
	REG_IP_ADDR_1,
	REG_IP_ADDR_2,
	REG_IP_ADDR_3,
	REG_MAC_ADDR_0,
	REG_MAC_ADDR_1,
	REG_MAC_ADDR_2,
	REG_MAC_ADDR_3,
	REG_MAC_ADDR_4,
	REG_MAC_ADDR_5,

	// Commands
	FX3_FIRST_COMMAND_SET = 0xE0,
	FX3_CMD_SAVE_FACTORY,
	FX3_CMD_ENABLE_TIMECODE = 0xE9,
	FX3_CMD_SET_RTC_TIME,
	FX3_CMD_REMOTE_INFO,
	FX3_CMD_UART_LAYOUT,
	FX3_CMD_UART_BAUD_RATE,
	FX3_CMD_PIP_SWAP,
	FX3_CMD_PIP_MODE,
	FX3_CMD_SAVE_CONFIG,
	FX3_CMD_SET_ANALOG_INPUT = 0xF9,
	FX3_CMD_SET_DMA_AUDIO,
	FX3_CMD_SET_DMA_VIDEO,
	FX3_CMD_LOW_POWER,
	FX3_CMD_PROGB = 0xFE,
	FX3_CMD_RESET
};

enum FPGAStatRegister
{
	Version = 0,
	LineRatesLSB,
	LineRatesMSB,
	FixedAB,
};

struct VideoFormat
{
	unsigned char Locked;
	unsigned short ActiveLinesF0;
	unsigned short ActivesPixel;
	unsigned short FrameRate;
};

struct SPIEEPROMLocation
{
	enum SPILocationDesc Destination;
	unsigned long FirstPage;
	unsigned long LastPage;
	unsigned long FirstSector;
	unsigned long LastSector;
	unsigned long Size;
	unsigned long FirstAddressActive;
	unsigned long LocationStartAddress;
};

static struct SPIEEPROMLocation FX3FirmwareInM25P80 =
{
	FX3,
	M25P80_FX3_FIRST_PAGE, 
	M25P80_FX3_LAST_PAGE, 
	M25P80_FX3_FIRST_SECTOR, 
	M25P80_FX3_LAST_SECTOR, 
	M25P80_FX3_SIZE, 
	M25P80_FX3_START_ADDRESS, 
	M25P80_FX3_START_ADDRESS
};

static struct SPIEEPROMLocation FPGABitStreamInM25P80 = 
{
	FPGA, 
	M25P80_FPGA_FIRST_PAGE, 
	M25P80_FPGA_LAST_PAGE, 
	M25P80_FPGA_FIRST_SECTOR, 
	M25P80_FPGA_LAST_SECTOR, 
	M25P80_FPGA_SIZE, 
	M25P80_FPGA_START_ADDRESS, 
	M25P80_FPGA_START_ADDRESS
};


static struct SPIEEPROMLocation EDIDInM25P80 = 
{
	EDID, 
	M25P80_EDID_FIRST_PAGE, 
	M25P80_EDID_LAST_PAGE, 
	M25P80_EDID_FIRST_SECTOR, 
	M25P80_EDID_LAST_SECTOR, 
	M25P80_EDID_SIZE, 
	M25P80_EDID_START_ADDRESS,
	M25P80_EDID_START_ADDRESS  
};

unsigned char MessageID(void);



enum ErrorCode GetFx3Version(unsigned char *Major, unsigned char *Minor);
enum ErrorCode isFx3Initialize(unsigned char *isInit);
enum ErrorCode GetFPGAVersion(unsigned char *Major, unsigned char *Minor);
enum ErrorCode GetEDIDVersion(unsigned char *Version);
enum ErrorCode SPIFPGAProgram(char *filename);
enum ErrorCode SPIFX3Program(char *filename);
enum ErrorCode SPIEDIDProgram(char *filename);
enum ErrorCode SPIFPGAVerify(char *filename);
enum ErrorCode SPIEDIDVerify(char *filename);
enum ErrorCode SPIFX3Verify(char *filename);
enum ErrorCode SetFX3Register(unsigned char Register, unsigned char RegisterValue);
void FX3Reset(void);

enum ErrorCode GetInputVideoFormat(struct VideoFormat *format);
enum ErrorCode ReadDescriptor(unsigned char *Data);
enum ErrorCode WriteDescriptor(unsigned char *Data);
	
#ifdef	__cplusplus
}
#endif

#endif	/* HIDCOMMAND_H */


