﻿

Public Class HIDCommand


#Region "Constant"
    Public Const MAXIMUM_FX3_ERROR_CODE As Byte = ErrorCode.HID_NO_RESPONSE_OR_INVALID - 1
#End Region


#Region "Enums"
    Public Enum Command
        HID_RESERVED0_CMD = 0
        HID_RESERVED1_CMD
        HID_RESERVED2_CMD
        HID_RESERVED3_CMD
        HID_RESERVED4_CMD
        HID_RESERVED5_CMD
        HID_RESERVED6_CMD
        HID_RESERVED7_CMD
        HID_FX3_WRITE_CMD = 8
        HID_FX3_READ_CMD = 9
    End Enum
    Public Enum ErrorCode
        HID_ACK = 0
        HID_BUSY
        HID_BAD_CMD_NUM
        HID_BAD_LENGTH
        HID_BAD_ADDR
        HID_TOO_BAD
        HID_BAD_DATA_LENGTH
        HID_SPI_FAIL
        HID_I2C_NO_ANSWER 'the target I2C device did not responde
        HID_I2C_COMM_FAILED ' Teh I2C communication failed the reason are not given
        HID_NOT_SUPPORTED ' The current command is valid but module is removed from code with define
        HID_REG_ACCES_NOT_SUPPORTED
        HID_FPGA_NOT_CONFIGURED
        'NN FX3 error code must come before this line
        HID_NO_RESPONSE_OR_INVALID ' this a error code for PC only: No response for current primitive
        HID_PC_EXCEPTION ' this a error code for PC only: No response for current primitive
    End Enum

    Public Enum FX3Register
        VersionMajor
        VersionMinor
        SPIType
        FPGAVersion
        REG_Reserved
        REG_WIDTH_MSB
        REG_WIDTH_LSB
        REG_HEIGHT_MSB
        REG_HEIGHT_LSB
        REG_FRAME_RATE_MSB
        REG_FRAME_RATE_LSB
        REG_VIDEO_GENERAL      ' //bit 0 1 == video locked
        EDID_VERSION
    End Enum

    Public Structure VideoFormat
        Public Locked As Byte
        Public ActiveLinesF0 As UShort
        Public ActivesPixel As UShort
    End Structure
#End Region

#Region "Properties"
    Private _MessageID As Byte = 1
    Public ReadOnly Property MessageId As Byte
        Get
            Dim temp As Byte = _MessageID
            If _MessageID < 255 Then
                _MessageID += 1
            Else
                _MessageID = 1
            End If
            Return temp
        End Get
    End Property

    Private _Primitive() As Byte
#End Region

#Region "Constructor"
    Public Sub New()
        clear()
    End Sub
#End Region

#Region "methods Common"
    Public Sub clear()
        _Primitive = Nothing
    End Sub
#End Region

#Region "Generic command"

    ''' <summary>
    ''' Parse the answer from a get FX3 Register Command
    ''' </summary>
    ''' <param name="Answer">The array of byte receive from USB3 link</param>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Public Function ParseDeviceAnswer(ByVal Answer() As Byte, ByRef ReturnAnswerData() As Byte) As ErrorCode
        If Answer Is Nothing OrElse Answer.Length < 4 OrElse _Primitive Is Nothing OrElse _Primitive.Length < 3 Then
            Return ErrorCode.HID_NO_RESPONSE_OR_INVALID
        End If
        'Check Message ID
        If _Primitive(0) <> Answer(0) Then
            'Oops this not an answer from the current request
            Return ErrorCode.HID_NO_RESPONSE_OR_INVALID
        End If
        'Check the command it must be the same
        If Answer(1) <> _Primitive(1) Then
            'Oops this not an answer from the current request
            Return ErrorCode.HID_NO_RESPONSE_OR_INVALID
        End If


        If Answer(2) < 1 Then
            Return ErrorCode.HID_NO_RESPONSE_OR_INVALID
        End If


        'Error code expected
        If Answer(3) = ErrorCode.HID_ACK Then
            'Success
        ElseIf Answer(3) > MAXIMUM_FX3_ERROR_CODE Then
            'Invalid
            Return ErrorCode.HID_NO_RESPONSE_OR_INVALID
        Else
            'error
            Return CType(Answer(3), ErrorCode)
        End If
        Dim DataLen As Integer = Answer(2) - 1 ' ok remove the acknoledge byte

        'Ok we have an answer
        If DataLen Then
            'we have an answer to return
            ReDim ReturnAnswerData(DataLen - 1) ' Ok set to grab all data from device
            Array.Copy(Answer, 4, ReturnAnswerData, 0, DataLen)
        End If
        Return CType(Answer(3), ErrorCode)
    End Function
    ''' <summary>
    ''' Parse the answer from a get FX3 Register Command
    ''' </summary>
    ''' <param name="Answer">The array of byte receive from USB3 link</param>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Public Function ParseDeviceAnswer(ByVal Answer() As Byte) As ErrorCode
        Dim Data() As Byte = Nothing
        Return ParseDeviceAnswer(Answer, Data)
    End Function
#End Region

#Region "FX3 Registers"
    ''' <summary>
    ''' Build a Get FX3 Regiter command
    ''' </summary>
    ''' <param name="Register">The Register to get</param>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Public Function BuildGetFX3Register(ByVal Register As Byte) As Byte()
        ReDim _Primitive(3) '4 bytes primitive
        _Primitive(0) = MessageId ' Message ID; manage by the object
        _Primitive(1) = Command.HID_FX3_READ_CMD ' command
        _Primitive(2) = 1 ' Always length of 1
        _Primitive(3) = Register 'Register
        Return _Primitive
    End Function
    ''' <summary>
    ''' Build a Set FX3 Regiter command
    ''' </summary>
    ''' <param name="Register">The Register to Set</param>
    ''' <param name="Value">The value at wich to set the register</param>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Public Function BuildSetFX3Register(ByVal Register As Byte, ByVal Value As Byte) As Byte()
        ReDim _Primitive(4) '5 byte primitive
        _Primitive(0) = MessageId ' Message ID; manage by the object
        _Primitive(1) = Command.HID_FX3_WRITE_CMD ' command
        _Primitive(2) = 2 ' Always length of 2
        _Primitive(3) = Register 'Register
        _Primitive(4) = Value 'Register value
        Return _Primitive
    End Function
#End Region

End Class
